#include "t3216_puzzle_code.h"
#include "pins.h"
#include "neopixels.h"

// #define PIN_LED_RED 11
#define PIN_LED_WHITE 12
// #define PIN_LED_BLUE 13

//////////////////////// OTHER ///////////////////////

unsigned long previous_time = 0;

//////////////////////////////////////////////////////
//////////////////////// SETUP ///////////////////////
//////////////////////////////////////////////////////
void setup() {
  // //Only use Serial for testing
  // Serial.swap(1); //Use alternate TX/RX positions
  // Serial.begin(9600);

  //LED pins to output
  pinMode(PIN_LED_WHITE, OUTPUT);
  digitalWrite(PIN_LED_WHITE, HIGH);
  // pinMode(PIN_LED_RED, OUTPUT);
  // digitalWrite(PIN_LED_RED, HIGH);
  // pinMode(PIN_LED_BLUE, OUTPUT);
  // digitalWrite(PIN_LED_BLUE, HIGH);

  /////////// NEOPIXELS ///////////
  //Set-up the Neopixels - Using the tinyNeoPixel library
  pinMode(pin_NEO, OUTPUT);
  // strip.begin(); //For the AdaFruit_NeoPixel library
  strip.clear();
  strip.show(); // Initialize all pixels to 'off'
  // strip.setBrightness(map(40, 0, 100, 0, 255)); //Set the overall brightness - doesn't work if I want to use getPixelColor
  randomSeed(analogRead(0)); //Extra randomness

  ///////////// PINS /////////////
  //Start I2C
  board_0x20.begin();
  board_0x21.begin();
  //Set the I2C clock to 400kHz
  board_0x20.setClock(400000);
  board_0x21.setClock(400000);

  //Put all PCA9555D pins to INPUT PULLUP
  for (byte i = 0; i < 16; i++) {
    board_0x20.pinMode(i, INPUT_PULLUP);
    board_0x21.pinMode(i, INPUT_PULLUP);
  }//for i

  //Set the ATtiny puzzle pins to INPUT PULLUP
  for (byte i = 1; i <= 6; i++) {
    pinMode(i, INPUT_PULLUP);
  }//for i
  
  //Set an initial state of all pins to "nothing placed"
  for(byte p = 0; p < num_pieces; p++) {
    pin_state[p] = 1;
    pin_state_old[p] = 1;
  }//for p

  /////////// INITIAL SHOW ///////////

  //Show an initial burst
  HEXAGON center = {0, 0, 0};
  // initialLightBurst(center);
  hexagonRainbow(center, false);
  hexagonRainbow(center, true);

  // //Print the amount of free SRAM
  // Serial.println(freeRam());
}//void setup

//////////////////////////////////////////////////////
//////////////////////// LOOP ////////////////////////
//////////////////////////////////////////////////////
void loop() {
  ///////////// PINS /////////////
  //Get the current state of all pins
  checkPins();
  //Check which puzzles are placed
  checkPiecePlaced(true); //true to keep a piece once placed

  /////////// NEOPIXELS ///////////
  //Set all NeoPixels to off
  strip.clear();

  //Siplex noise color field
  colorField(false); //true to fill the entire board, not just puzzle piece locations

  ///////// PUZZLE PIECES /////////
  //Check if a new piece is placed
  for(byte p = 0; p < num_pieces; p++) {
    if(piece_matrix[p].piece_placed == true) {
      //If a new piece is placed, show a light burst
      int n = piece_matrix[p].center_id;
      HEXAGON center = {x_grid[n], y_grid[n], z_grid[n]};
      lightBurst(center);
      piece_matrix[p].piece_placed = !piece_matrix[p].piece_placed;
      pieces_placed += 1;
    }//if
  }//for p

  ///////////// BURST /////////////
  //If a new piece is placed, continuously show light bursts
  //starting from random puzzle locations
  if(pieces_placed >= num_pieces) {
    byte p = random(num_pieces);
    byte n = piece_matrix[p].center_id;
    HEXAGON center = {x_grid[n], y_grid[n], z_grid[n]};
    lightBurst(center);
  }//if

  /////////////////////////////////
  // Serial.println(millis() - previous_time);
  previous_time = millis();

  // //Print the amount of free SRAM
  // Serial.println(freeRam());
}//void loop

//////////////////////////////////////////////////////
////////////////// HELPER FUNCTIONS //////////////////
//////////////////////////////////////////////////////

//Returns the delta between top of the heap and bottom of the stack, i.e. the remaining amount of free SRAM
int freeRam() {
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}//int freeRam

//Mapping function that works with floating points
float mapf(float x, float in_min, float in_max, float out_min, float out_max) {
    return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}//function mapf
